require("utils.svg.parse")
local color = require("utils.io.colors")

files = love.filesystem.getDirectoryItems("tests")
for i, fn in ipairs(files) do
  files[i] = "tests/" .. fn
end
current = 1

canvas = nil
-- set the background color to something other than black
love.graphics.setBackgroundColor(128, 128, 128)

-- render svg to canvas
function render_svg(fn)
  t1 = os.clock()
  -- load file
  local sz = love.filesystem.read(fn)
  assert(sz, "could not read file:" .. fn)
  t2 = os.clock()
  -- parse, triangulate, etc
  shapes = svg.eval(sz)
  t3 = os.clock()
  -- let's start drawing
  canvas = love.graphics.newCanvas()
  love.graphics.setCanvas(canvas)
  for _, shape in ipairs(shapes) do
    -- todo: uses one color and alpha per shape
    local r,g,b = color.hex_to_rgb(shape.colors[1])
    local a = shape.alpha[1]*255
    love.graphics.setColor(r,g,b,a)
    -- draw triangles
    local v = shape.vertices
    for i = 1, #v, 6 do
      love.graphics.polygon('fill',v[i],v[i + 1], v[i + 2],v[i + 3], v[i + 4],v[i + 5])
    end
  end
  -- re-enable drawing to the main screen
  love.graphics.setCanvas()
  t4 = os.clock()
end

function love.load()
  render_svg(files[current])
end

-- nothing too heavy
function love.draw()
  if canvas then
    love.graphics.draw(canvas)
    love.graphics.setColor(255,255,255,255)
    love.graphics.print(files[current], 0, 0)
    love.graphics.print("reading: " .. t2-t1 .. " ms", 0, 20)
    love.graphics.print("parsing: " .. t3-t2 .. " ms", 0, 40)
    love.graphics.print("drawing: " .. t4-t3 .. " ms", 0, 60)
    love.graphics.print("arrow keys: iterate files", 0, 80)
  end
end

function love.keypressed(k)
  if k == "left" then
    current = current - 1
    if current < 1 then
      current = #files
    end
  elseif k == "right" then
    current = current + 1
    if current > #files then
      current = 1
    end
  end
  render_svg(files[current])
end